/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions.config.uisettings;


import android.content.Context;

import com.foxit.sdk.PDFViewCtrl;
import com.foxit.sdk.common.Renderer;
import com.foxit.uiextensions.R;
import com.foxit.uiextensions.UIExtensionsManager;

import java.util.HashMap;

public class UISettingsManager {

    private PDFViewCtrl mPDFViewCtrl;
    private Context mContext;
    private UISettingsConfig mUISettings;
    private UIExtensionsManager mUIExtensionsManager;

    public UISettingsManager(Context context, PDFViewCtrl pdfViewCtrl) {
        mPDFViewCtrl = pdfViewCtrl;
        mContext = context;
        mUIExtensionsManager = (UIExtensionsManager) (pdfViewCtrl.getUIExtensionsManager());
        mUISettings = mUIExtensionsManager.getConfig().uiSettings;
    }

    public void setUISettings() {
        //displayMode
        String displayMode = mUISettings.displayMode;
        if (!"Reflow".equals(displayMode)) {
            HashMap<String, Integer> diaplayMaps = arrays2Map(R.array.displayModes);
            Integer displayModeVaule = diaplayMaps.get(displayMode);
            if (displayModeVaule != null) {
                mPDFViewCtrl.setPageLayoutMode(displayModeVaule);
            }
        }
        //zoomMode
        String zoomMode = mUISettings.zoomMode;
        HashMap<String, Integer> zoomMaps = arrays2Map(R.array.zoomModes);
        Integer zoomModeVaule = zoomMaps.get(zoomMode);
        if (zoomModeVaule != null) {
            mPDFViewCtrl.setZoomMode(zoomModeVaule);
        }
        //colorMode
        String colorMode = mUISettings.colorMode;
        if ("Map".equals(colorMode)) {
            mPDFViewCtrl.setMappingModeForegroundColor(mUISettings.mapForegroundColor);
            mPDFViewCtrl.setMappingModeBackgroundColor(mUISettings.mapBackgroundColor);
            mPDFViewCtrl.setColorMode(Renderer.e_ColorModeMapping);
        }
        //highlightForm
        boolean highlightForm = mUISettings.highlightForm;
        mUIExtensionsManager.enableFormHighlight(highlightForm);
        //highlightFormColor
        int highlightFormColor = mUISettings.highlightFormColor;
        mUIExtensionsManager.setFormHighlightColor(highlightFormColor);
        //highlightLink
        boolean highlightLink = mUISettings.highlightLink;
        mUIExtensionsManager.enableLinkHighlight(highlightLink);
        //highlightLinkColor
        int highlightLinkColor = mUISettings.highlightLinkColor;
        mUIExtensionsManager.setLinkHighlightColor(highlightLinkColor);

        /*---------annotations--------*/
        //continuouslyAdd
        boolean continuouslyAdd = mUISettings.annotations.continuouslyAdd;
        mUIExtensionsManager.setContinueAddAnnot(continuouslyAdd);
    }

    private HashMap<String, Integer> arrays2Map(int resid) {
        String[] strings = mContext.getResources().getStringArray(resid);
        HashMap<String, Integer> map = new HashMap<>();
        for (String str : strings) {
            String[] items = str.split("#");
            map.put(items[1], Integer.parseInt(items[0]));
        }
        return map;
    }

}
