/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions.modules.signature;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.drawable.ColorDrawable;
import android.view.Gravity;
import android.view.ViewGroup;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;

import com.foxit.sdk.PDFViewCtrl;
import com.foxit.uiextensions.R;
import com.foxit.uiextensions.UIExtensionsManager;
import com.foxit.uiextensions.pdfreader.config.ReadStateConfig;
import com.foxit.uiextensions.utils.AppDisplay;
import com.foxit.uiextensions.utils.LayoutConfig;


class SignatureMixListPopup {
    protected interface ISignatureListEvent {
        void onSignatureListDismiss();
    }

    private SignatureListPicker mListPicker;
    private PopupWindow mPopupWindow;
    private ViewGroup mParent;
    private Context mContext;
    private ISignatureListEvent mListEvent;

    protected SignatureMixListPopup(Context context, ViewGroup parent, final PDFViewCtrl pdfViewCtrl, SignatureFragment.SignatureInkCallback inkCallback) {
        this.mParent = parent;
        this.mContext = context;

        mListPicker = new SignatureListPicker(context, parent, pdfViewCtrl, inkCallback);
        mListPicker.init(new SignatureListPicker.ISignListPickerDismissCallback() {
            @Override
            public void onDismiss(boolean isShowAnnotMenu) {
                mPopupWindow.dismiss();
            }
        });

        if (mPopupWindow == null) {
            mPopupWindow = new PopupWindow(mListPicker.getRootView(), RelativeLayout.LayoutParams.MATCH_PARENT,
                    RelativeLayout.LayoutParams.MATCH_PARENT, true);
        }
        mPopupWindow.setBackgroundDrawable(new ColorDrawable(0xFFFFFFFF));
        mPopupWindow.setAnimationStyle(R.style.View_Animation_RtoL);
        mPopupWindow.setOnDismissListener(new PopupWindow.OnDismissListener() {
            @Override
            public void onDismiss() {
                if (mListPicker.getBaseItemsSize() == 0) {
                    ((UIExtensionsManager) pdfViewCtrl.getUIExtensionsManager()).setCurrentToolHandler(null);
                    ((UIExtensionsManager) pdfViewCtrl.getUIExtensionsManager()).changeState(ReadStateConfig.STATE_NORMAL);
                }
                mListPicker.dismiss();
                ((UIExtensionsManager) pdfViewCtrl.getUIExtensionsManager()).getMainFrame().hideMaskView();
                if (mListEvent != null) {
                    mListEvent.onSignatureListDismiss();
                }
            }
        });
    }

    protected void show() {
        mListPicker.loadData();

        int width = mParent.getWidth();
        int height = mParent.getHeight();

        if (AppDisplay.getInstance(mContext).isPad()) {
            float scale = LayoutConfig.RD_PANEL_WIDTH_SCALE_V;
            if (width > height) {
                scale = LayoutConfig.RD_PANEL_WIDTH_SCALE_H;
            }
            width = (int) (AppDisplay.getInstance(mContext).getScreenWidth() * scale);
        }
        mPopupWindow.setWidth(width);
        mPopupWindow.setHeight(height);
        mPopupWindow.showAtLocation(mParent, Gravity.RIGHT | Gravity.TOP, 0, 0);
    }

    protected void update() {
        int rootWidth = mParent.getWidth();
        int rootHeight = mParent.getHeight();

        boolean bVertical = mContext.getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT;
        int width, height;
        if (bVertical) {
            height = Math.max(rootWidth, rootHeight);
            width = Math.min(rootWidth, rootHeight);
        } else {
            height = Math.min(rootWidth, rootHeight);
            width = Math.max(rootWidth, rootHeight);
        }
        if (AppDisplay.getInstance(mContext).isPad()) {
            float scale = LayoutConfig.RD_PANEL_WIDTH_SCALE_V;
            if (width > height) {
                scale = LayoutConfig.RD_PANEL_WIDTH_SCALE_H;
            }
            width = (int) (AppDisplay.getInstance(mContext).getScreenWidth() * scale);
        }
        mPopupWindow.update(width, height);
    }

    protected PopupWindow getPopWindow() {
        return mPopupWindow;
    }

    protected void setSignatureListEvent(ISignatureListEvent event) {
        mListEvent = event;
    }

}
