/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.home;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;

import com.foxit.App;
import com.foxit.pdfreader.PDFReaderActivity;
import com.foxit.pdfreader.fragment.BaseFragment;
import com.foxit.uiextensions.controls.toolbar.BaseBar;
import com.foxit.uiextensions.controls.toolbar.impl.BaseItemImpl;
import com.foxit.uiextensions.home.IHomeModule;
import com.foxit.uiextensions.home.local.LocalModule;
import com.foxit.uiextensions.modules.connectpdf.account.AccountModule;
import com.foxit.uiextensions.utils.AppFileUtil;

import java.util.Map;

public class MainActivity extends AppCompatActivity implements ActivityCompat.OnRequestPermissionsResultCallback{

    private boolean mLicenseValid = false;
    public static final int REQUEST_EXTERNAL_STORAGE = 1;
    private static final String[] PERMISSIONS_STORAGE = {
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE
    };
    String filter = App.FILTER_DEFAULT;
    View.OnClickListener mSingleClickListener;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mLicenseValid = App.instance().checkLicense();
        if(!mLicenseValid)
            return;
        this.requestWindowFeature(Window.FEATURE_NO_TITLE);

        if (getIntent() != null) {
            filter = getIntent().getAction();
        }

        initTabsButton(App.instance().getLocalModule(filter), this);

        if (Build.VERSION.SDK_INT >= 23) {
            int permission = ContextCompat.checkSelfPermission(this.getApplicationContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE);
            if (permission != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(this, PERMISSIONS_STORAGE, REQUEST_EXTERNAL_STORAGE);
            }
        }

        App.instance().copyGuideFiles(App.instance().getLocalModule(filter));
        App.instance().getLocalModule(filter).setFileItemEventListener(new IHomeModule.onFileItemEventListener() {
            @Override
            public void onFileItemClicked(String fileExtra, String filePath) {
                onFileSelected(fileExtra, filePath);
            }
        });
        AccountModule.getInstance().onCreate(this, savedInstanceState);

        View view = App.instance().getLocalModule(filter).getContentView(this.getApplicationContext());
        ViewGroup parent = (ViewGroup) view.getParent();
        if (parent != null) {
            parent.removeView(view);
        }
        setContentView(view);

        handleIntent(getIntent());
    }

    @Override
    protected void onDestroy() {
        if (mLicenseValid) {
            App.instance().unloadLocalModule(filter);
            AccountModule.getInstance().onDestroy(this);
        }

        mSingleClickListener = null;
        super.onDestroy();
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        setIntent(intent);
        handleIntent(intent);
    }

    private void handleIntent(Intent intent) {
        if (intent!= null) {
            String path = AppFileUtil.getFilePath(this, intent, IHomeModule.FILE_EXTRA);
            if (path != null) {
                onFileSelected(IHomeModule.FILE_EXTRA, path);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (mLicenseValid && requestCode == REQUEST_EXTERNAL_STORAGE
                && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            App.instance().copyGuideFiles(App.instance().getLocalModule(filter));
            App.instance().getLocalModule(filter).updateStoragePermissionGranted();
            initTabsButton(App.instance().getLocalModule(filter), this);
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    private void onFileSelected(String fileExtra, String filePath) {
        Intent intent = new Intent();
        intent.putExtra(fileExtra, filePath);
        intent.putExtra("filter", filter);
        intent.setClass(this.getApplicationContext(), PDFReaderActivity.class);

        if (App.instance().isMultiTab()) {
            intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        }
        this.startActivity(intent);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK && App.instance().isMultiTab()) {
            Intent launcherIntent = new Intent(Intent.ACTION_MAIN);
            launcherIntent.addCategory(Intent.CATEGORY_HOME);
            startActivity(launcherIntent);
            return true;
        }
        App.instance().onBack();
        return super.onKeyDown(keyCode, event);
    }

    private void initTabsButton(LocalModule localModule, final Activity activity) {
        BaseItemImpl singleMultiBtn = (BaseItemImpl) App.instance().getTabsButton(filter);
        if (singleMultiBtn != null) {
            if (singleMultiBtn.getContentView().getParent() == null) {
                localModule.getTopToolbar().addView(singleMultiBtn, BaseBar.TB_Position.Position_RB);
            }
            return;
        }
        singleMultiBtn = new BaseItemImpl(this.getApplicationContext());
        App.instance().setTabsButton(filter, singleMultiBtn);

        if (App.instance().isMultiTab()) {
            singleMultiBtn.setImageResource(R.drawable.rd_multi_tab_selector);
            singleMultiBtn.setId(R.id.rd_multi_tab);
        } else {
            singleMultiBtn.setImageResource(R.drawable.rd_single_tab_selector);
            singleMultiBtn.setId(R.id.rd_single_tab);
        }

        final BaseItemImpl finalSingleMultiBtn = singleMultiBtn;
        singleMultiBtn.setOnClickListener(mSingleClickListener = new View.OnClickListener() {
            @Override
            public void onClick(final View v) {
                String readerMode = !App.instance().isMultiTab() ? getString(R.string.fx_tabs_reader_mode) : getString(R.string.fx_single_reader_mode);
                String msg = getString(R.string.fx_swith_reader_mode_toast, readerMode);
                String title = "";
                Dialog dialog = new AlertDialog.Builder(activity).setCancelable(true).setTitle(title)
                        .setMessage(msg)
                        .setPositiveButton(getString(R.string.fx_string_yes),
                                new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        if (v.getId() == R.id.rd_single_tab) {
                                            finalSingleMultiBtn.setImageResource(R.drawable.rd_multi_tab_selector);
                                            finalSingleMultiBtn.setId(R.id.rd_multi_tab);
                                            App.instance().setMultiTabFlag(true);
                                        } else if (v.getId() == R.id.rd_multi_tab) {
                                            finalSingleMultiBtn.setImageResource(R.drawable.rd_single_tab_selector);
                                            finalSingleMultiBtn.setId(R.id.rd_single_tab);
                                            App.instance().setMultiTabFlag(false);

                                            FragmentManager mFragmentManager = App.instance().getTabsManager(filter).getFragmentManager();
                                            if (mFragmentManager != null) {
                                                FragmentTransaction fragmentTransaction = mFragmentManager.beginTransaction();
                                                for (Map.Entry<String, BaseFragment> entry : App.instance().getTabsManager(filter).getFragmentMap().entrySet()) {
                                                    fragmentTransaction.remove(entry.getValue());
                                                }
                                                fragmentTransaction.commitAllowingStateLoss();
                                            }
                                            if (App.instance().getTabsManager(filter).getCurrentFragment() != null &&
                                                    App.instance().getTabsManager(filter).getCurrentFragment().getActivity() != null) {
                                                App.instance().getTabsManager(filter).getCurrentFragment().getActivity().finish();
                                            }
                                            App.instance().getTabsManager(filter).setCurrentFragment(null);
                                            App.instance().getTabsManager(filter).clearFragment();
                                        }
                                        App.instance().getTabsManager(filter).setFragmentManager(null);
                                        App.instance().getMultiTabView(filter).resetData();

                                        dialog.dismiss();
                                    }
                                }).setNegativeButton(getString(R.string.fx_string_no),
                                new DialogInterface.OnClickListener() {

                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        dialog.dismiss();
                                    }
                                }).create();
                dialog.show();
            }
        });
        localModule.getTopToolbar().addView(singleMultiBtn, BaseBar.TB_Position.Position_RB);
    }
}
