/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.pdfreader;

import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.view.KeyEvent;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.LinearLayout;

import com.foxit.App;
import com.foxit.home.MainActivity;
import com.foxit.home.R;
import com.foxit.pdfreader.fragment.BaseFragment;
import com.foxit.pdfreader.fragment.EmptyViewFragment;
import com.foxit.pdfreader.fragment.PDFReaderFragment;
import com.foxit.uiextensions.ToolHandler;
import com.foxit.uiextensions.UIExtensionsManager;
import com.foxit.uiextensions.home.IHomeModule;
import com.foxit.uiextensions.modules.signature.SignatureToolHandler;
import com.foxit.uiextensions.utils.AppFileUtil;
import com.foxit.uiextensions.utils.AppTheme;



public class PDFReaderActivity extends FragmentActivity implements UIExtensionsManager.OnFinishListener {

    private FragmentManager mFragmentManager;

    private String filePath;
    String filter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        AppTheme.setThemeFullScreen(this);
        AppTheme.setThemeNeedMenuKey(this);
        setContentView(R.layout.activity_reader);
        filter = getIntent().getStringExtra("filter");
        if (App.instance().getTabsManager(filter).getFragmentManager() == null || !App.instance().isMultiTab()) {
            mFragmentManager = getSupportFragmentManager();
            App.instance().getTabsManager(filter).setFragmentManager(mFragmentManager);
        }
        if(!App.instance().checkLicense()) {
            openEmptyView();
        } else {
            getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
            if (App.instance().isMultiTab()) {
                App.instance().getMultiTabView(filter).registerTabEventListener(mTabEventListener);
            }
            openDocument(getIntent());
        }
    }

    /**
     * open a Doc View use Fragment
     */
    private void openDocView(boolean isRemoveCurFragment){
        filePath = App.instance().getTabsManager(filter).getFilePath();
        mFragmentManager = App.instance().getTabsManager(filter).getFragmentManager();
        FragmentTransaction fragmentTransaction = mFragmentManager.beginTransaction();
        PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(filePath);
        if (fragment == null) {
            fragment = new PDFReaderFragment();
            if (App.instance().isMultiTab()) {
                App.instance().getTabsManager(filter).addFragment(filePath, fragment);
            }
            fragment.setPath(filePath);
            fragment.setOnFinishListener(this);
            fragment.filter = filter;
        } else {
            if (App.instance().isMultiTab()) {
                if (!fragment.isOpenSuccess) {
                    App.instance().getTabsManager(filter).removeFragment(filePath);
                    fragmentTransaction.remove(fragment);

                    fragment = new PDFReaderFragment();
                    App.instance().getTabsManager(filter).addFragment(filePath, fragment);
                    fragment.setPath(filePath);
                    fragment.setOnFinishListener(this);
                    fragment.filter = filter;
                }
            }
        }

        if (!fragment.isAdded()) {
            fragmentTransaction.add(R.id.reader_container, fragment);
        }

        if (App.instance().getTabsManager(filter).getCurrentFragment() != null &&
                !App.instance().getTabsManager(filter).getCurrentFragment().equals(fragment)) {
            if (isRemoveCurFragment) {
                fragmentTransaction.remove(App.instance().getTabsManager(filter).getCurrentFragment());
            } else {
                fragmentTransaction.hide(App.instance().getTabsManager(filter).getCurrentFragment());
            }
        }

        fragmentTransaction.show(fragment).commitAllowingStateLoss();

        if (App.instance().isMultiTab()) {
            App.instance().getTabsManager(filter).setCurrentFragment(fragment);
        }
    }

    /**
     * when App license is valid, it should open a empty view use Fragment also.
     */
    private void openEmptyView(){
        FragmentTransaction fragmentTransaction = mFragmentManager.beginTransaction();
        fragmentTransaction.replace(R.id.reader_container, new EmptyViewFragment());
        fragmentTransaction.commitAllowingStateLoss();
    }

    private void backToMainActivity() {
        Intent intent = new Intent();
        intent.setClass(getApplicationContext(), MainActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(intent);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        BaseFragment currentFrag = (BaseFragment) getSupportFragmentManager().findFragmentById(R.id.reader_container);

        if (App.instance().isMultiTab()) {

            if (keyCode == KeyEvent.KEYCODE_BACK){
                currentFrag = App.instance().getTabsManager(filter).getCurrentFragment();
                if (currentFrag.mUiExtensionsManager != null
                        && currentFrag.mUiExtensionsManager.backToNormalState()){
                    return true;
                } else {
                    backToMainActivity();
                    return true;
                }
            } else {
                return super.onKeyDown(keyCode, event);
            }
        }

        if (currentFrag != null && currentFrag.mUiExtensionsManager != null && currentFrag.mUiExtensionsManager.onKeyDown(this, keyCode, event))
            return true;
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onFinish() {
        if (App.instance().isMultiTab()) {
            BaseFragment currentFrag = App.instance().getTabsManager(filter).getCurrentFragment();
            if (currentFrag != null && !currentFrag.isOpenSuccess) {
                backToMainActivity();
            }
        } else {
            finish();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (App.instance().isMultiTab()) {
            App.instance().getMultiTabView(filter).unregisterTabEventListener(mTabEventListener);
        }

        mTabEventListener = null;
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        filter = intent.getStringExtra("filter");
        mFragmentManager = App.instance().getTabsManager(filter).getFragmentManager();
        if (mFragmentManager == null) {
            mFragmentManager = getSupportFragmentManager();
            App.instance().getTabsManager(filter).setFragmentManager(mFragmentManager);
        }

        setIntent(intent); // import. otherwise, it will get the old intent.

        openDocument(intent);
    }

    private void openDocument(Intent intent) {
        filePath = App.instance().getTabsManager(filter).getFilePath();
        String oldPath = filePath;
        PDFReaderFragment oldFragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(oldPath);
        if (oldPath != null && oldFragment != null && oldFragment.isOpenSuccess){
            oldFragment.mUiExtensionsManager.stopHideToolbarsTimer();
        }

        filePath = AppFileUtil.getFilePath(this, intent, IHomeModule.FILE_EXTRA);
        App.instance().getTabsManager(filter).setFilePath(filePath);
        PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(filePath);
        boolean needReset = oldPath != null && !filePath.equals(oldPath) && fragment != null && fragment.isOpenSuccess;
        if(!App.instance().checkLicense())
            openEmptyView();
        else {
            openDocView(false);
        }

        if (needReset) {
            resetTabView(true);
        }
    }

    private MultiTabView.ITabEventListener mTabEventListener = new MultiTabView.ITabEventListener() {
        @Override
        public void onTabChanged(MultiTabView.TabInfo oldTabInfo, MultiTabView.TabInfo newTabInfo) {
            PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(oldTabInfo.tabTarget);
            fragment.mUiExtensionsManager.stopHideToolbarsTimer();
            changeViewerState(fragment);
            filePath = newTabInfo.tabTarget;
            App.instance().getTabsManager(filter).setFilePath(filePath);

            PDFReaderFragment newfragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(filePath);
            newfragment.mUiExtensionsManager.getDocumentManager().resetActionCallback();

            openDocView(false);
            resetTabView(false);

            newfragment.mUiExtensionsManager.getMainFrame().showToolbars();
            newfragment.mUiExtensionsManager.resetHideToolbarsTimer();
        }

        @Override
        public void onTabRemoved(final MultiTabView.TabInfo removedTab, final MultiTabView.TabInfo showTab) {
            final PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(removedTab.tabTarget);
            if (removedTab.tabTarget.equals(App.instance().getTabsManager(filter).getFilePath())) {
                changeViewerState(fragment);

                fragment.doClose(new BaseFragment.IFragmentEvent() {
                    @Override
                    public void onRemove() {
                        App.instance().getMultiTabView(filter).removeTab(removedTab);
                        if (showTab != null) {
                            filePath = showTab.tabTarget;
                            App.instance().getTabsManager(filter).setFilePath(filePath);

                            PDFReaderFragment newfragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(filePath);
                            newfragment.mUiExtensionsManager.getDocumentManager().resetActionCallback();

                            openDocView(true);
                            resetTabView(false);
                            App.instance().getTabsManager(filter).getFragmentMap().remove(removedTab.tabTarget);
                            App.instance().getMultiTabView(filter).refreshTopBar(showTab.tabTarget);
                        } else {
                            App.instance().getTabsManager(filter).setFilePath(null);

                            // only one tab
                            removeFragment(fragment);
                            App.instance().getTabsManager(filter).setCurrentFragment(null);
                            App.instance().getTabsManager(filter).clearFragment();

                            Intent intent = new Intent();
                            intent.setClass(getApplicationContext(), MainActivity.class);
                            intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                            startActivity(intent);
                        }
                    }
                });
            } else {
                fragment.doClose(new BaseFragment.IFragmentEvent() {
                    @Override
                    public void onRemove() {
                        App.instance().getMultiTabView(filter).removeTab(removedTab);
                        removeFragment(fragment);
                        App.instance().getTabsManager(filter).getFragmentMap().remove(removedTab.tabTarget);
                        App.instance().getMultiTabView(filter).refreshTopBar(showTab.tabTarget);
                    }
                });
            }
        }
    };

    private void resetTabView(boolean needRefresh) {
        if (needRefresh) {
            App.instance().getMultiTabView(filter).refreshTopBar(App.instance().getTabsManager(filter).getFilePath());
        }
        PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(App.instance().getTabsManager(filter).getFilePath());
        int h = fragment.mUiExtensionsManager.getMainFrame().getTopToolbar().getContentView().getHeight();
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 2 * h / 3);
        params.topMargin = -10;
        ViewGroup parent = (ViewGroup) App.instance().getMultiTabView(filter).getTabView().getParent();
        if (parent != null) {
            parent.removeView(App.instance().getMultiTabView(filter).getTabView());
        }
        fragment.mUiExtensionsManager.getMainFrame().addSubViewToTopBar(App.instance().getMultiTabView(filter).getTabView(), 1, params);
    }

    private void changeViewerState(PDFReaderFragment fragment) {
        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager) fragment.getPdfViewCtrl().getUIExtensionsManager();
        uiExtensionsManager.triggerDismissMenuEvent();
        uiExtensionsManager.getDocumentManager().setCurrentAnnot(null);
        uiExtensionsManager.exitPanZoomMode();
        ToolHandler toolHandler = uiExtensionsManager.getCurrentToolHandler();
        if (toolHandler != null) {
            uiExtensionsManager.setCurrentToolHandler(null);
            if (toolHandler instanceof SignatureToolHandler) {
                fragment.getPdfViewCtrl().invalidate();
            }
        }
    }

    public void removeFragment(BaseFragment fragment) {
        mFragmentManager = App.instance().getTabsManager(filter).getFragmentManager();
        FragmentTransaction fragmentTransaction = mFragmentManager.beginTransaction();
        fragmentTransaction.remove(fragment).commitAllowingStateLoss();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        BaseFragment currentFrag = (BaseFragment) getSupportFragmentManager().findFragmentById(R.id.reader_container);
        if (App.instance().isMultiTab()) {
            currentFrag = App.instance().getTabsManager(filter).getCurrentFragment();
            if (currentFrag != null && currentFrag.isOpenSuccess) {
                App.instance().getMultiTabView(filter).refreshTopBar(currentFrag.getPath());
            }
        }

        if (currentFrag == null) return;
        if (currentFrag.mUiExtensionsManager != null) {
            currentFrag.mUiExtensionsManager.onConfigurationChanged(this, newConfig);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        BaseFragment currentFrag = (BaseFragment) getSupportFragmentManager().findFragmentById(R.id.reader_container);
        if (App.instance().isMultiTab()) {
            currentFrag = App.instance().getTabsManager(filter).getCurrentFragment();
        }
        if (currentFrag == null) return;
        currentFrag.handleActivityResult(requestCode, resultCode, data);
    }
}
