/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.pdfreader.fragment;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import com.foxit.App;
import com.foxit.home.MainActivity;
import com.foxit.home.R;
import com.foxit.sdk.PDFViewCtrl;
import com.foxit.sdk.common.Constants;
import com.foxit.sdk.pdf.PDFDoc;
import com.foxit.uiextensions.IPDFReader;
import com.foxit.uiextensions.Module;
import com.foxit.uiextensions.UIExtensionsManager;
import com.foxit.uiextensions.config.Config;
import com.foxit.uiextensions.controls.dialog.AppDialogManager;
import com.foxit.uiextensions.controls.dialog.MatchDialog;
import com.foxit.uiextensions.controls.dialog.UITextEditDialog;
import com.foxit.uiextensions.controls.dialog.fileselect.UIFolderSelectDialog;
import com.foxit.uiextensions.home.local.LocalModule;
import com.foxit.uiextensions.modules.dynamicxfa.DynamicXFAModule;
import com.foxit.uiextensions.utils.AppDmUtil;
import com.foxit.uiextensions.utils.AppFileUtil;
import com.foxit.uiextensions.utils.AppResource;
import com.foxit.uiextensions.utils.AppUtil;
import com.foxit.uiextensions.utils.UIToast;

import java.io.File;
import java.io.FileFilter;
import java.io.InputStream;

import static com.foxit.sdk.common.Constants.e_ErrSuccess;

public class PDFReaderFragment extends BaseFragment {

    private final static String TAG = PDFReaderFragment.class.getSimpleName();
    private PDFViewCtrl pdfViewerCtrl;
    private AlertDialog mSaveAlertDlg;
    private ProgressDialog mProgressDlg;
    private String mSavePath = null;
    private String mProgressMsg = null;
    private boolean isSaveDocInCurPath = false;
    private String currentFileCachePath;
    private IFragmentEvent mFragmentEvent = null;
    private String mDocPath;
    private boolean isCloseDocAfterSaving = false;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (mUiExtensionsManager != null) {
            return mUiExtensionsManager.getContentView();
        }
        InputStream stream = getActivity().getApplicationContext().getResources().openRawResource(R.raw.uiextensions_config);
        Config config = new Config(stream);

        pdfViewerCtrl = new PDFViewCtrl(getActivity().getApplicationContext());
        mUiExtensionsManager = new UIExtensionsManager(getActivity().getApplicationContext(), pdfViewerCtrl, config);

        if (App.instance().isMultiTab()) {
            pdfViewerCtrl.registerDocEventListener(mDocEventListener);
        }

        pdfViewerCtrl.setUIExtensionsManager(mUiExtensionsManager);
        pdfViewerCtrl.setAttachedActivity(getActivity());
        mUiExtensionsManager.setAttachedActivity(getActivity());
        mUiExtensionsManager.registerModule(App.instance().getLocalModule(filter)); // use to refresh file list

        mUiExtensionsManager.onCreate(getActivity(), pdfViewerCtrl, savedInstanceState);
        mUiExtensionsManager.openDocument(getPath(), null);
        mUiExtensionsManager.setOnFinishListener(getOnFinishListener());
        setName("");

        mUiExtensionsManager.setBackEventListener(mBackEventListener);
        return  mUiExtensionsManager.getContentView();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (App.instance().isMultiTab()) {
            mUiExtensionsManager.getMainFrame().removeSubViewFromTopBar(App.instance().getMultiTabView(filter).getTabView());
            if (pdfViewerCtrl != null) {
                pdfViewerCtrl.unregisterDocEventListener(mDocEventListener);
            }

            mUiExtensionsManager.setBackEventListener(null);
        }

        mDocEventListener = null;
        mBackEventListener = null;
    }

    public PDFViewCtrl getPdfViewCtrl() {
        return pdfViewerCtrl;
    }

    PDFViewCtrl.IDocEventListener mDocEventListener = new PDFViewCtrl.IDocEventListener() {
        @Override
        public void onDocWillOpen() {

        }

        @Override
        public void onDocOpened(PDFDoc document, int errCode) {
            isOpenSuccess = errCode == Constants.e_ErrSuccess;
            if (App.instance().isMultiTab() && errCode == Constants.e_ErrSuccess) {
                mDocPath = pdfViewerCtrl.getFilePath();

                if (!getPath().equals(mDocPath)){
                    //Remove the same file that has been opened
                    PDFReaderFragment fragment = (PDFReaderFragment) App.instance().getTabsManager(filter).getFragmentMap().get(mDocPath);
                    if (fragment != null){
                        App.instance().getMultiTabView(filter).getHistoryFileNames().remove(mDocPath);

                        FragmentManager mFragmentManager = App.instance().getTabsManager(filter).getFragmentManager();
                        FragmentTransaction fragmentTransaction = mFragmentManager.beginTransaction();
                        fragmentTransaction.remove(fragment).commitAllowingStateLoss();

                        App.instance().getTabsManager(filter).getFragmentMap().remove(mDocPath);
                    }

                    App.instance().getTabsManager(filter).getFragmentMap().remove(getPath());
                    App.instance().getTabsManager(filter).addFragment(mDocPath,PDFReaderFragment.this);
                    App.instance().getTabsManager(filter).setFilePath(mDocPath);

                    int index = App.instance().getMultiTabView(filter).getHistoryFileNames().indexOf(getPath());
                    App.instance().getMultiTabView(filter).getHistoryFileNames().set(index, mDocPath);

                    setPath(mDocPath);
                    App.instance().getMultiTabView(filter).refreshTopBar(mDocPath);
                } else {
                    App.instance().getMultiTabView(filter).refreshTopBar(getPath());
                }

                int h = mUiExtensionsManager.getMainFrame().getTopToolbar().getContentView().getHeight();
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 2 * h / 3);
                params.topMargin = -10;
                ViewGroup parent = (ViewGroup) App.instance().getMultiTabView(filter).getTabView().getParent();
                if (parent != null) {
                    parent.removeView(App.instance().getMultiTabView(filter).getTabView());
                }
                mUiExtensionsManager.getMainFrame().addSubViewToTopBar(App.instance().getMultiTabView(filter).getTabView(), 1, params);
            }
        }

        @Override
        public void onDocWillClose(PDFDoc document) {

        }

        @Override
        public void onDocClosed(PDFDoc document, int errCode) {
            if(isSaveDocInCurPath) {
                File file = new File(currentFileCachePath);
                File docFile = new File(mDocPath);
                if (file.exists()) {
                    docFile.delete();
                    if (!file.renameTo(docFile))
                        UIToast.getInstance(getContext()).show(getContext().getString(R.string.fx_save_file_failed));
                }else{
                    UIToast.getInstance(getContext()).show(getContext().getString(R.string.fx_save_file_failed));
                }

            }

            if (errCode == e_ErrSuccess && isSaveDocInCurPath) {
                updateThumbnail(mSavePath);
            }

            if (mFragmentEvent != null) {
                mFragmentEvent.onRemove();
            }
        }

        @Override
        public void onDocWillSave(PDFDoc document) {

        }

        @Override
        public void onDocSaved(PDFDoc document, int errCode) {
            if (errCode == e_ErrSuccess && !isSaveDocInCurPath) {
                updateThumbnail(mSavePath);
            }

            if (isCloseDocAfterSaving){
                closeAndSaveDoc(mFragmentEvent);
            }
        }
    };


    private IPDFReader.BackEventListener mBackEventListener = new IPDFReader.BackEventListener() {
        @Override
        public boolean onBack() {
            if (App.instance().isMultiTab()) {
                Intent intent = new Intent();
                intent.setClass(getContext(), MainActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                startActivity(intent);
                return true;
            }
            return false;
        }
    };

    public void doClose(final IFragmentEvent callback) {
        if (pdfViewerCtrl == null) return;

        if (pdfViewerCtrl.isDynamicXFA()) {
            DynamicXFAModule dynamicXFAModule = (DynamicXFAModule) mUiExtensionsManager.getModuleByName(Module.MODULE_NAME_DYNAMICXFA);
            if (dynamicXFAModule != null && dynamicXFAModule.getCurrentXFAWidget() != null) {
                dynamicXFAModule.setCurrentXFAWidget(null);
            }
        }

        if (pdfViewerCtrl.getDoc() == null || !mUiExtensionsManager.getDocumentManager().isDocModified()) {
            mProgressMsg = getContext().getString(R.string.fx_string_closing);
            closeAndSaveDoc(callback);
            return;
        }

        final boolean hideSave = !pdfViewerCtrl.isDynamicXFA() && !mUiExtensionsManager.canModifyContents();
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        String[] items;
        if (hideSave) {
            items = new String[]{
                    getContext().getString(R.string.rv_back_save_to_new_file),
                    getContext().getString(R.string.rv_back_discard_modify),
            };
        } else {
            items = new String[]{
                    getContext().getString(R.string.rv_back_save_to_original_file),
                    getContext().getString(R.string.rv_back_save_to_new_file),
                    getContext().getString(R.string.rv_back_discard_modify),
            };
        }

        builder.setItems(items, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                if (hideSave) {
                    which += 1;
                }
                switch (which) {
                    case 0: // save
                        isCloseDocAfterSaving = true;
                        String userSavePath = mUiExtensionsManager.getSavePath();
                        if (userSavePath != null && userSavePath.length() > 0 && !userSavePath.equalsIgnoreCase(mDocPath)) {
                            File userSaveFile = new File(userSavePath);
                            File defaultSaveFile = new File(mDocPath);
                            if (userSaveFile.getParent().equalsIgnoreCase(defaultSaveFile.getParent())) {
                                isSaveDocInCurPath = true;
                                mSavePath = userSavePath;
                            } else {
                                isSaveDocInCurPath = false;
                            }
                            pdfViewerCtrl.saveDoc(userSavePath, mUiExtensionsManager.getSaveDocFlag());
                        } else {
                            isSaveDocInCurPath = true;
                            pdfViewerCtrl.saveDoc(getCacheFile(), mUiExtensionsManager.getSaveDocFlag());
                        }
                        isSaveDocInCurPath = true;
                        mProgressMsg = getContext().getString(R.string.fx_string_saving);
                        mFragmentEvent = callback;
                        showProgressDialog();
                        break;
                    case 1: // save as
                        mProgressMsg = getContext().getString(R.string.fx_string_saving);
                        onSaveAsClicked();
                        break;
                    case 2: // discard modify
                        mProgressMsg = getContext().getString(R.string.fx_string_closing);
                        closeAndSaveDoc(callback);
                        break;
                    default:
                        break;
                }
                dialog.dismiss();
                mSaveAlertDlg = null;
            }

            void showInputFileNameDialog(final String fileFolder) {
                String newFilePath = fileFolder + "/" + AppFileUtil.getFileName(pdfViewerCtrl.getFilePath());
                final String filePath = AppFileUtil.getFileDuplicateName(newFilePath);
                final String fileName = AppFileUtil.getFileNameWithoutExt(filePath);

                final UITextEditDialog rmDialog = new UITextEditDialog(getActivity());
                rmDialog.setPattern("[/\\:*?<>|\"\n\t]");
                rmDialog.setTitle(AppResource.getString(getContext(),  R.string.fx_string_saveas));
                rmDialog.getPromptTextView().setVisibility(View.GONE);
                rmDialog.getInputEditText().setText(fileName);
                rmDialog.getInputEditText().selectAll();
                rmDialog.show();
                AppUtil.showSoftInput(rmDialog.getInputEditText());

                rmDialog.getOKButton().setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        rmDialog.dismiss();
                        String inputName = rmDialog.getInputEditText().getText().toString();
                        String newPath = fileFolder + "/" + inputName;
                        newPath += ".pdf";
                        File file = new File(newPath);
                        if (file.exists()) {
                            showAskReplaceDialog(fileFolder,newPath);
                        } else {
                            isCloseDocAfterSaving = true;
                            mSavePath = newPath;
                            pdfViewerCtrl.saveDoc(newPath, mUiExtensionsManager.getSaveDocFlag());
                            mFragmentEvent = callback;
                            showProgressDialog();
                        }
                    }
                });
            }

            void showAskReplaceDialog(final String fileFolder,final String newPath) {
                final UITextEditDialog rmDialog = new UITextEditDialog(getActivity());
                rmDialog.setTitle(AppResource.getString(getContext(), R.string.fx_string_saveas));
                rmDialog.getPromptTextView().setText(AppResource.getString(getContext(), R.string.fx_string_filereplace_warning));
                rmDialog.getInputEditText().setVisibility(View.GONE);
                rmDialog.show();

                rmDialog.getOKButton().setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        rmDialog.dismiss();
                        isCloseDocAfterSaving = true;
                        mSavePath = newPath;
                        if (newPath.equalsIgnoreCase(pdfViewerCtrl.getFilePath())) {
                            isSaveDocInCurPath = true;
                            pdfViewerCtrl.saveDoc(getCacheFile(), mUiExtensionsManager.getSaveDocFlag());
                        } else {
                            isSaveDocInCurPath = false;
                            pdfViewerCtrl.saveDoc(newPath, mUiExtensionsManager.getSaveDocFlag());
                        }
                        mFragmentEvent = callback;
                        showProgressDialog();
                    }
                });

                rmDialog.getCancelButton().setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        rmDialog.dismiss();
                        showInputFileNameDialog(fileFolder);
                    }
                });
            }

            void onSaveAsClicked() {
                final UIFolderSelectDialog dialog = new UIFolderSelectDialog(getActivity());
                dialog.setFileFilter(new FileFilter() {
                    @Override
                    public boolean accept(File pathname) {
                        return !(pathname.isHidden() || !pathname.canRead()) && !pathname.isFile();
                    }
                });
                dialog.setTitle(AppResource.getString(getContext(), R.string.fx_string_saveas));
                dialog.setButton(MatchDialog.DIALOG_OK | MatchDialog.DIALOG_CANCEL);
                dialog.setListener(new MatchDialog.DialogListener() {
                    @Override
                    public void onResult(long btType) {
                        if (btType == MatchDialog.DIALOG_OK) {
                            String fileFolder = dialog.getCurrentPath();
                            showInputFileNameDialog(fileFolder);
                        }
                        dialog.dismiss();
                    }

                    @Override
                    public void onBackClick() {
                    }
                });
                dialog.showDialog();
            }
        });

        mSaveAlertDlg = builder.create();
        mSaveAlertDlg.setCanceledOnTouchOutside(true);
        mSaveAlertDlg.show();
    }

    private String getCacheFile(){
        mSavePath = pdfViewerCtrl.getFilePath();
        File file = new File(mSavePath);
        String dir = file.getParent()+"/";
        while(file.exists()){
            currentFileCachePath = dir+ AppDmUtil.randomUUID(null)+".pdf";
            file = new File(currentFileCachePath);
        }
        return currentFileCachePath;
    }

    private void showProgressDialog() {
        if (mProgressDlg == null && getActivity() != null) {
            mProgressDlg = new ProgressDialog(getActivity());
            mProgressDlg.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            mProgressDlg.setCancelable(false);
            mProgressDlg.setIndeterminate(false);
            mProgressDlg.setMessage(mProgressMsg);
            AppDialogManager.getInstance().showAllowManager(mProgressDlg, null);
        }
    }

    private void closeAndSaveDoc(IFragmentEvent callback) {
        showProgressDialog();
        pdfViewerCtrl.closeDoc();
        mFragmentEvent = callback;
    }

    private void updateThumbnail(String path) {
        LocalModule module = (LocalModule) mUiExtensionsManager.getModuleByName(Module.MODULE_NAME_LOCAL);
        if (module != null && path != null) {
            module.updateThumbnail(path);
        }
    }

    @Override
    public void handleActivityResult(int requestCode, int resultCode, Intent data) {
        mUiExtensionsManager.handleActivityResult(getActivity(), requestCode, resultCode, data);
    }
}
